#!/bin/sh
if [ "`id -u`" != 0 ]; then
	echo "$0 must be run as root."
	exit 1
fi
[ ! -x ./gen-cd.sh ] && cd `dirname $0`
SRC=`pwd`

PATH=$PATH:/sbin:/usr/sbin
export PATH

[ -z "$VERSION" ] && VERSION="7.2"
[ -z "$HOSTTYPE" ] && HOSTTYPE=i386
WE_HAVE_LOTS_OF_SPACE="" # Simply documenting its existance. ;)

RPM_DIRS="$SRC/$HOSTTYPE /mnt/redhat/latest/$HOSTTYPE"

# Kernel modules we aren't including because of boot disk size issues:
# Network: airo airo_cs atp ewrk3 hamachi hp hp-plus hp100 lne390 sk98lin
#          de4x5 defxx depca dgrs dmfe eth16i ac3200 natsemi rcpci sk_g16
#          smc91c92_cs smc-ultra at1700 e100 e1000 e2100
#          ne3210 pcnet32 sb1000 xircom_tulip_cb
# SCSI: eata eata_dma eata_pio seagate sim710 tmscsim ncr53c8xx NCR53c406a
#       aha152x aha1542 aha1740 fdomain gdth imm in2000 ips osst sym53c416
#	psi240i qla1280
# Other/Unknown: fmvj18x_cs 3w-xxxx yellowfin winbond-840 sundance
MODULES="cramfs cdrom ide-cd 3c501 3c503 3c505 3c507 3c509 3c515 3c574_cs \
 3c589_cs 3c59x 8139too 82596 8390 acenic cs89x0 de600 de620 \
 eepro eepro100 eexpress epic100 es3210 lance ne \
 ne2k-pci smc-ultra32 starfire tlan sis900 via-rhine \
 53c7,8xx AM53C974 BusLogic DAC960 a100u2w aacraid advansys \
 aic7xxx atp870u cciss cb_enabler \
 cpqarray ds dmx3191d dtc fdomain initio megaraid pas16 pci2000 pci2220i ppa \
 qla2x00 qlogicfas qlogicfc qlogicisp \
 sym53c8xx t128 u14-34f ultrastor wd7000 \
 scsi_mod sd_mod st xor \
 fat g_NCR5380 i82365 ide-tape \
 ide-cs initio linear lockd loop msdos ncr53c8xx \
 ray_cs netwave_cs nfs pcmcia_core pcnet_cs qlogic_cs raid0 raid1 raid5 \
 tulip usb-ohci usbserial usb-storage usb-uhci usbcore usbkbd \
 tcic yenta_socket input vfat wavelan_cs wd7000 wvlan_cs xirc2ps_cs \
 xircom_cb sunrpc \
 ext3 jbd"

DESTDIR=`pwd`/CD-IMAGE

LIBS="e2fsprogs glibc libtermcap ncurses openssl procps readline zlib dev \
      utempter popt db1"
PROGS="bash bzip2 fileutils gawk grub rpm sh-utils textutils cpio grep \
	findutils net-tools ncftp gzip joe lilo modutils mount nc iputils \
	sed strace tar pax telnet util-linux SysVinit parted pinfo psmisc \
	man screen file dosfstools raidtools tcpdump bind-utils mt-st \
	rsync less links openssh openssh-clients pump vim reiserfs-utils \
	gpart e3 dump pciutils bdflush eject initscripts rungetty-rescue \
	portmap nfs-utils ppp minicom mkbootdisk glibc-common mailx"


# Functions...
status() {
	echo -e "\033[1;33m*** $1 ***\033[0;39m"
}
error() {
	echo -e "\033[1;31m$1\033[0;39m"
}
cleanup() {
	# Used only for trap
	[ -d $SRC/modules/bootdisk ] && umount $SRC/modules/bootdisk
	[ -d $SRC/modules/initrd ] && umount $SRC/modules/initrd
	error Aborted
	exit 1
}

trap cleanup 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 \
	24 25 26 27 28 29 30 31

# Sanity checks...
status "RUNNING SANITY CHECKS"
for i in $RPM_DIRS; do
	if [ ! -d $i ]; then
		error "RPM directory $i doesn't exist!"
		exit 1
	fi
done

# Anaconda requires LOTS of stuff to build!
for i in pump pump-devel kudzu kudzu-devel pciutils pciutils-devel bzip2 \
	bzip2-devel e2fsprogs e2fsprogs-devel python python-devel db3 \
	db3-devel gtk+ gtk+-devel gnome-libs gnome-libs-devel \
	syslinux dosfstools wget; do
	[ ! rpm -q $i &>/dev/null ] && rpm -i `findrpm $i*`
done

rm -rf $DESTDIR
mkdir -p $DESTDIR

# -----------------------------------------------------------------------------
status "BUILDING ANACONDA"
if [ -n "$USE_ANACONDA_7_0" ]; then
	if [ ! -d $SRC/anaconda-7.0 ]; then
	  rm -fr anaconda-7.0
	  tar xjf anaconda-7.0.tar.bz2
	  patch -p0 -b -z .resc < anaconda-7.0.patch
	  perl -pi -e "s/-g -O2/-O2 -Os/g" anaconda-7.0/collage/Makefile anaconda-7.0/gnome-map/Makefile anaconda-7.0/isys/modutils/configure
	  perl -pi -e "s/-g -O/-O2 -Os/g" anaconda-7.0/libfdisk/Makefile
	fi

	cd anaconda-7.0
	make || exit 10
	strip -R .comment --strip-unneeded loader/loader-rescue
	strip -R .comment --strip-unneeded loader/init
elif [ -n "$USE_ANACONDA_7_1" ]; then
	if [ ! -d $SRC/anaconda-7.1 ]; then
	  rm -fr anaconda-7.1
	  tar xjf anaconda-7.1.tar.bz2
	  patch -p0 -b -z .resc < anaconda-7.1.patch
	  find . -name Makefile |xargs perl -pi -e "s/-O2/-O2 -Os/g;s/-g -O2/-O2 -Os/g;s/-g -O/-O2 -Os/g;s/-ggdb/-O2 -Os/g;s/-O2 -g/-O2 -Os/g;s/-O -g/-O2 -Os/g;s/-g/-O2 -Os/g;s/-Os -Os/-Os/g"
	  find . -name Makefile |xargs perl -pi -e "s/-Os -Os/-Os/g"
	fi
	cd anaconda-7.1
	make || exit 10
	strip -R .comment --strip-unneeded loader/loader-rescue
	strip -R .comment --strip-unneeded loader/init
else
	if [ ! -d $SRC/anaconda ]; then
	  rm -fr anaconda
	  tar xjf anaconda-7.2.tar.bz2
	  patch -p0 -b -z .resc < anaconda-7.2.patch
	  find . -name Makefile |xargs perl -pi -e "s/-O2/-O2 -Os/g;s/-g -O2/-O2 -Os/g;s/-g -O/-O2 -Os/g;s/-ggdb/-O2 -Os/g;s/-O2 -g/-O2 -Os/g;s/-O -g/-O2 -Os/g;s/-g/-O2 -Os/g;s/-Os -Os/-Os/g"
	  find . -name Makefile |xargs perl -pi -e "s/-Os -Os/-Os/g"
	fi
	cd anaconda
	make || exit 10
	strip -R .comment --strip-unneeded loader/loader-rescue
	strip -R .comment --strip-unneeded loader/init
fi

# -----------------------------------------------------------------------------
status "BUILDING BOOT DISK IMAGE"
cd ../boot

nod() {
	mknod $1 $2 $3 $4
	chmod $5 $1
	chown $6 $1
	chgrp $7 $1
}
findrpm() {
	# find a file (or set of files) in RPM_DIRS...
	local PKG=""
	for i in $RPM_DIRS; do
		if [ -z "$PKG" -a -d "$i" ]; then
			pushd $i &>/dev/null
			# Don't use test -e; $1 might contain wildcards...
			PKG=`ls |egrep $1 2>/dev/null`
			[ ! -z $PKG ] && PKG="$i/$PKG"
			popd &>/dev/null
		fi
	done
	if [ -z "$PKG" ]; then
		error "No package matching $PKG found!"
		exit 1
	fi
	echo $PKG
}

rm -rf bootdisk initrd modules boot modules.cgz modules.dep
KERNEL=`findrpm kernel-2*$HOSTTYPE.rpm |sed -e "s,.*kernel-,,g" -e "s,.i386.rpm,,"`
dd if=/dev/zero of=rescue.img bs=1k count=2880
dd if=/dev/zero of=initrd.img bs=1k count=4096
mkdosfs rescue.img
mke2fs -F -m0 initrd.img
mkdir -p modules bootdisk initrd
mount -tvfat -oloop rescue.img bootdisk
mount -text2 -oloop initrd.img initrd
cd modules || exit 1
rpm2cpio `findrpm kernel-$KERNEL.$HOSTTYPE.rpm` | cpio -id --quiet
rm -rf sbin boot
mkdir ${KERNEL}
for i in $MODULES; do
	MODULE=`find lib/modules/${KERNEL} -name $i.o`
	if [ -z "$MODULE" ]; then
		error "Can't find $i module"
	else
		cp -fHpR $MODULE ${KERNEL}
	fi
done
cd ${KERNEL}
depmod -inq * >../../modules.dep
cd ..
rm -rf sbin boot
find ${KERNEL} | cpio -Hcrc -o > modules.cpio
gzip -9f modules.cpio
mv modules.cpio.gz ../modules.cgz
cd ..
perl -pi -e 's,\.o,,g; s,^\n,,g; s,\\\n,,g; s,[\t ]+, ,g' modules.dep
rm -rf modules
cd initrd
mkdir -p dev etc/terminfo/l modules proc sbin tmp var/state
rpm2cpio `findrpm kernel-pcmcia-cs-*.$HOSTTYPE.rpm` | cpio -id --quiet
rm -rf etc/rc.d etc/sysconfig/* sbin/* usr
ln -sf /sbin/init linuxrc
ln -sf sbin bin
ln -sf loader sbin/insmod
ln -sf loader sbin/modprobe
ln -sf loader sbin/rmmod
ln -sf /usr/bin/sh sbin/sh
ln -sf /proc/mounts etc/mtab
ln -sf /tmp/ld.so.conf etc
ln -sf /tmp var/state/xkb
cp -af /usr/share/terminfo/l/linux etc/terminfo/l
mv -f ../modules.cgz modules
mv -f ../modules.dep modules
if [ -n "$USE_ANACONDA_7_0" ]; then
	cp -af ../../anaconda-7.0/loader/init-rescue bin/init
	cp -af ../../anaconda-7.0/loader/loader-rescue bin/loader
	cp -af ../../anaconda-7.0/fonts/fonts-i386 etc/fonts.cgz
	cp -af ../../anaconda-7.0/keymaps/keymaps-i386 etc/keymaps.gz
	cp -af ../../anaconda-7.0/lang-table etc
	cp -af ../../anaconda-7.0/loader/loader.tr etc
	perl -p -e 's/^#.*\n//; s/^\s*\n//' ../../anaconda-7.0/loader/module-info > modules/module-info
	../../anaconda-7.0/utils/trimpcitable ${MODULES} < ../../anaconda-7.0/kudzu/pcitable > modules/pcitable
	#cp -af ../../anaconda-7.0/isys/pci/pcitable modules
elif [ -n "$USE_ANACONDA_7_1" ]; then
	cp -af ../../anaconda-7.1/loader/init-rescue bin/init
	cp -af ../../anaconda-7.1/loader/loader-rescue bin/loader
	cp -af ../../anaconda-7.1/fonts/fonts-i386 etc/fonts.cgz
	cp -af ../../anaconda-7.1/keymaps/keymaps-i386 etc/keymaps.gz
	cp -af ../../anaconda-7.1/lang-table etc
	cp -af ../../anaconda-7.1/loader/loader.tr etc
	perl -p -e 's/^#.*\n//; s/^\s*\n//' ../../anaconda-7.1/loader/module-info > modules/module-info
	$SRC/trimpcitable ${MODULES} < ${SRC}/pcitable > modules/pcitable
else
	cp -af ../../anaconda/loader/init-rescue bin/init
	cp -af ../../anaconda/loader/loader-rescue bin/loader
	cp -af ../../anaconda/fonts/fonts-i386 etc/fonts.cgz
	cp -af ../../anaconda/keymaps/keymaps-i386 etc/keymaps.gz
	cp -af ../../anaconda/lang-table etc
	cp -af ../../anaconda/loader/loader.tr etc
	perl -p -e 's/^#.*\n//; s/^\s*\n//' ../../anaconda/loader/module-info > modules/module-info
	$SRC/trimpcitable ${MODULES} < ${SRC}/pcitable > modules/pcitable
fi
echo "root::0:0:root:/:/bin/bash" >etc/passwd
cd dev
nod agpgart c	10	175		0664	root	root
nod console c	5	1		0600	root	root
for i in 0 1 2 3 4 5 6 7; do
  nod fb$i c	29 	`expr $i \* 32`	0600	root	root
done
for i in 0 1 2 3 4 5 6 7 8 9; do
  nod tty$i c	4	$i		0600	root	tty
done
for i in 0 1 2 3; do
  nod ttyS$i c	4	`expr $i + 64`	0600	root	root
done
mkdir input
nod input/mice c 13	63		0664	root	root
nod kbd c	11	0		0644	root	root
for i in `seq 0 31`; do
  nod md$i b	9	$i		0666	root	root
done
nod mem c	1	1		0600	root	root
nod null c	1	3		0666	root	root
nod openprom c 10	139		0644	root	root
nod psaux c	10	1		0644	root	root
nod ptmx c	5	2		0666	root	root
mkdir pts
nod ptyp0 c	2	0		0644	root	root
nod ram b	1	1		0640	root	root
nod sunmouse c	10	6		0644	root	root
nod ttyp0 c	3	0		0644	root	root
nod zero c	1	5		0666	root	root
cd ../..
dd if=/dev/zero of=initrd/nothing &>/dev/null || : # zero out unused space
rm -f initrd/nothing
umount initrd
gzip -9f initrd.img
mv -f initrd.img.gz bootdisk/initrd.img
cp -f syslinux.cfg bootdisk/syslinux.cfg || echo syslinux.cfg not found in $(pwd)
for i in `ls *.{msg,MSG} 2>/dev/null`; do
	sed "s/@VERSION@/$VERSION/g" < $i > bootdisk/$i
done
rpm2cpio `findrpm kernel-$KERNEL.$HOSTTYPE.rpm` | cpio -id --quiet \*vmlinuz\*
mv boot/vmlinuz-$KERNEL bootdisk/vmlinuz
umount bootdisk
syslinux rescue.img
rm -rf bootdisk initrd boot

# -----------------------------------------------------------------------------
status "INSTALLING APPLICATIONS"

cd $DESTDIR 
# Create the basic directory layout... 

mkdir -p boot bin dev etc lib/gconv lib/rpm proc sbin \
	usr/share/info usr/share/ssl/misc var/cache initrd
ln -sf var/tmp tmp
ln -sf var/mnt mnt
ln -sf ../bin usr/bin
ln -sf ../sbin usr/sbin
ln -sf ../lib usr/lib

# Install stuff
for i in $LIBS $PROGS; do
	(rpm2cpio `findrpm $i-[.0123456789]*.$HOSTTYPE.rpm` || echo "Error in $i RPM (`findrpm $i-[.0123456789]*.$HOSTTYPE.rpm`)" >/dev/stderr) | cpio -id --quiet
done
# gawk and gzip need special treatment: since /usr/bin/gawk is a symlink
# to /bin/gawk and /usr/bin is a symlink to /bin, we end up with /bin/gawk
# being a symlink to itself
# Also, push in dmidecode
rm -f bin/gawk bin/awk bin/gzip bin/gunzip
TMP=`mktemp -d /tmp/workaroundXXXXXX`
mkdir -p $TMP
pushd $TMP
(rpm2cpio `findrpm gawk-[.0123456789]*.$HOSTTYPE.rpm` || echo "Error in gawk RPM (`findrpm gawk-[.0123456789]*.$HOSTTYPE.rpm`)" >/dev/stderr) |cpio -id --quiet
(rpm2cpio `findrpm gzip-[.0123456789]*.$HOSTTYPE.rpm` || echo "Error in gawk RPM (`findrpm gawk-[.0123456789]*.$HOSTTYPE.rpm`)" >/dev/stderr) |cpio -id --quiet
[ ! -e dmidecode.c ] && wget http://people.redhat.com/arjanv/dmidecode.c
gcc -O2 -Os -o bin/dmidecode dmidecode.c
popd
cp -af $TMP/bin/{awk,gawk,gzip,gunzip,dmidecode} bin
rm -rf $TMP
# Our mailer should get in
cp -af $SRC/bin/mail bin

KERNEL=`findrpm kernel-2.?.*.$HOSTTYPE.rpm |sed -e "s,.*kernel-,,g" -e "s,.$HOSTTYPE.rpm,,"`
# Install kernel modules
rpm2cpio `findrpm kernel-$KERNEL.$HOSTTYPE.rpm` | cpio -id --quiet \*lib/modules\*
cd lib/modules/$KERNEL

if [ -z "$WE_HAVE_LOTS_OF_SPACE" ]; then       
	rm -rf kernel/drivers/char/agp kernel/drivers/char/drm kernel/drivers/char/joystick kernel/drivers/media kernel/drivers/sound kernel/drivers/telephony kernel/drivers/video kernel/fs/autofs kernel/fs/autofs4 kernel/fs/binfmt_* kernel/fs/nls kernel/drivers/net/appletalk kernel/drivers/isdn kernel/net/khttpd
fi

cd ..
depmod -C $SRC/etc/modules.conf -inq $(find ./$KERNEL -name \*.o ) | sed -e 's,\./,/lib/modules/,g' > $KERNEL/modules.dep

# Ugly, but necessary... Split modules.dep into modules.dep, modules.isapnpmap,
# modules.pcimap and friends...
cd $KERNEL
LINES=`cat modules.dep |wc -l`; LINES=`expr $LINES`
PCI=`grep -n "pci module" modules.dep|sed -e "s/:.*//"`; PCI=`expr $PCI - 2`
head -n $PCI modules.dep >modules.dep.real
tail -n `expr $LINES - $PCI` modules.dep >modules.pcimap

LINES=`cat modules.pcimap |wc -l`; LINES=`expr $LINES`
ISA=`grep -n "isapnp module" modules.pcimap|sed -e "s/:.*//"`; ISA=`expr $ISA - 1`
head -n $ISA modules.pcimap >modules.pcimap.real
tail -n `expr $LINES - $ISA` modules.pcimap >modules.isapnpmap

LINES=`cat modules.isapnpmap |wc -l`; LINES=`expr $LINES`
USB=`grep -n "usb module" modules.isapnpmap|sed -e "s/:.*//"`; USB=`expr $USB - 1`
head -n $USB modules.isapnpmap >modules.isapnpmap.real
tail -n `expr $LINES - $USB` modules.isapnpmap >modules.usbmap
mv -f modules.isapnpmap.real modules.isapnpmap
mv -f modules.pcimap.real modules.pcimap
mv -f modules.dep.real modules.dep
cd ..


if [ -z "$WITH_CRAMFS" ]; then
	tar cjf ../modules-$KERNEL.tar.bz2 $KERNEL
	rm -rf $KERNEL
else
	rm -f ../modules-$KERNEL.tar.bz2 
fi

cd ../..

# Do ldconfig's job
#for i in lib/*.so.*.* usr/lib/*.so.*.*; do
#	VER=`echo $i |sed -e 's/.*.so\.//'`
#	MAJOR_VER=`echo $VER |sed -e 's/\..*//'`
#	TARGET=`echo $i |sed -e "s/\.so\..*/.so.$MAJOR_VER/"`
#	[ ! -e $TARGET ] && ln -sf `basename $i` $TARGET
#done
/sbin/ldconfig -r ./ || exit 10

# Remove unneeded/unwanted stuff that's in the packages we want...
KILLME_LIB='librt*'
KILLME_BIN='login kbdrate c_rehash captoinfo catchsegv chsh cksum chfn col \
        colcrt colrm column comm csplit cytune ddate expand factor \
        fmt fold gencat gendiff getconf getent getopt glibcbug \
        groups gzexe iconv infocmp infotocap ipcrm ipcs \
        keytab-lilo last lastb lddlibc4 locale localedef logger logname \
        look mcookie memusage memusagestat mesg mtrace ncftpbatch ncftpls \
        ncftpbookmarks ncftpls ncftpput ncftpget newgrp nl oldps openssl \
        paste pathchk pcprofiledump pinky pr printenv printf ptx \
        raw rename rev rpcgen script setfdprm sprof \
        ssh-keygen sum tac tack tic tload toe tsort tty tzselect \
        ul unexpand uptime users utmpdump uuidgen vmstat w wall \
        whereis who whoami write xtrace znew ssh-agent ssh-add bash2 \
        bashbug gawk-* vimtutor nsupdate man2html db1_dump'
KILLME_SBIN='clockdiff insmod.static sulogin \
	sysctl iptunnel kbdrate readprofile \
	rpcinfo slattach sln tunelp \
	vigr vipw zic netconfig genksyms zdump cfdisk'
KILLME_DEV='atari* amiga* audio* bttv* capi* cui* dcxx* input/js* ippp* \
	ir* isdn* js*'

rm -rf usr/share/locale usr/share/i18n usr/share/doc usr/share/terminfo 
rm -rf usr/share/zoneinfo usr/games usr/src usr/share/awk usr/libexec usr/include var/* 
rm -rf usr/share/catman/cat1/CA.pl.1
rm -rf usr/share/man/man1/CA.pl.1
for i in `ls usr/share/man | grep -v 'man[01456789n]'`; do
	rm -rf usr/share/man/$i
done

cd lib
rm -rf locale more.help libnss1* getopt libform* libhistory* \
	libpanel* libthread_db* libmemusage* nslookup.help *.a *.la \
	libNoVer* libBroken* libSeg* libmenu*
cd ../etc
rm -rf rpc X11 skel cron.* fdprm bashrc \
	ld.so.conf localtime  
cd ../sbin
[ -n "$KILLME_SBIN" ] && rm -rf $KILLME_SBIN
cd ../bin
[ -n "$KILLME_BIN" ] && rm -rf $KILLME_BIN
cd ../dev
[ -n "$KILLME_DEV" ] && rm -rf $KILLME_DEV
cd ../lib
[ -n "$KILLME_LIB" ] && rm -rf $KILLME_LIB
cd ..

# -----------------------------------------------------------------------------
if [ -z "$WITH_CRAMFS" ]; then
	status "COMPRESSING BINARIES"
	if [ -x $SRC/upx ]; then
		$SRC/upx -9 bin/*
	else
		error "upx not found, skipping"
	fi
fi

# -----------------------------------------------------------------------------
status "PRE-FORMATTING MAN PAGES"

cd usr/share/man
for i in $KILLME_SBIN $KILLME_BIN; do
	rm -f */$i.* 2>/dev/null || :
done
for i in *; do
	m=`echo $i |sed -e "s/man/cat/"`
	mkdir -p $DESTDIR/usr/share/catman/$m
	cd $DESTDIR/usr/share/man/$i
	for j in *; do
		echo -n "."
		man $DESTDIR/usr/share/man/$i/$j >$DESTDIR/usr/share/catman/$m/`echo $j |sed -e 's/\.gz//'` 2>/dev/null
	done
done
echo
# -----------------------------------------------------------------------------
if [ -z "$WITH_CRAMFS" ]; then
	status "COMPRESSING MAN PAGES"
	for i in $DESTDIR/usr/share/catman/*/*; do bzip2 -9f $i; done
fi

rm -rf $DESTDIR/usr/share/man
cd $DESTDIR

# -----------------------------------------------------------------------------
status "ADDING SCRIPTS, ETC."
# Move in some new stuff
mkdir -p $DESTDIR/etc/rc.d/init.d $DESTDIR/var
#gcc -O2 -fomit-frame-pointer -fno-exceptions -fno-rtti -mcpu=i686 -Os -o $DESTDIR/sbin/microgetty $SRC/microgetty.c || exit 1

#gcc -O2 -fomit-frame-pointer -fno-exceptions -fno-rtti -mcpu=i686 -Os -o $DESTDIR/sbin/halt $SRC/halt.c || exit 1
#ln -sf /sbin/halt $DESTDIR/sbin/reboot
#ln -sf /sbin/halt $DESTDIR/sbin/shutdown
#ln -sf /sbin/halt $DESTDIR/sbin/poweroff

ln -fs libcrypto.so.0.9.6 /usr/lib/libcrypto.so.1

#cp -f $SRC/etc/{halt,reboot} $DESTDIR/etc
ln -fs rc.d/init.d $DESTDIR/etc/init.d
ln -fs rc.d/rc.local $DESTDIR/etc/rc.local
ln -fs rc.d/rc.sysinit $DESTDIR/etc/rc.sysinit
ln -fs /tmp/ld.so.conf $DESTDIR/etc/ld.so.conf
ln -fs /tmp/modem $DESTDIR/dev/modem
ln -fs /tmp/cdrom $DESTDIR/dev/cdrom
cp -f $SRC/etc/modules.conf $DESTDIR/etc/modules.conf
cp -f $SRC/etc/rc.d/init.d/halt $DESTDIR/etc/rc.d/init.d
echo '#' >$DESTDIR/etc/fstab
echo 'NETWORKING=yes' > $DESTDIR/etc/sysconfig/network
cp -f $SRC/etc/inittab $DESTDIR/etc
cp -f $SRC/etc/issue $DESTDIR/etc
cp -f $SRC/etc/rc.d/rc.sysinit $DESTDIR/etc/rc.d
cp -f $SRC/sbin/mountall $DESTDIR/sbin
cp -f $SRC/etc/termcap $DESTDIR/etc
cp -f $SRC/etc/screenrc $DESTDIR/etc
echo -e "localhost\t127.0.0.1" >$DESTDIR/etc/hosts
echo "root::0:0:root:/tmp:/bin/bash" >$DESTDIR/etc/passwd
echo "nobody::99:99::/tmp:/bin/false" >>$DESTDIR/etc/passwd
echo "root::0:root" >$DESTDIR/etc/group
echo "nobody::99:nobody" >>$DESTDIR/etc/group
echo -e "retries 2\ntimeout 4" >$DESTDIR/etc/pump.conf
echo "telnet	23/tcp" >$DESTDIR/etc/services
touch $DESTDIR/etc/.rescue
cat >$DESTDIR/etc/man.config <<EOF
FHS
MANSECT 1:8:2:3:4:5:6:7:9:n
.gz             /bin/gunzip -c
.bz2            /usr/bin/bzip2 -c -d
.z
.Z              /bin/zcat
EOF
ln -sf /tmp/resolv.conf $DESTDIR/etc
#ln -sf /tmp/adjtime $DESTDIR/etc
ln -sf /proc/mounts $DESTDIR/etc/mtab
cp -f $SRC/etc/profile $DESTDIR/etc
sed "s/@VERSION@/$VERSION/g" < $SRC/README > $DESTDIR/README
ln -sf libtermcap.so.2.0.8 $DESTDIR/lib/libtermcap.so.2
mkdir -p $DESTDIR/usr/share/terminfo
tic -o $DESTDIR/usr/share/terminfo $DESTDIR/etc/termcap
ln -sf less $DESTDIR/bin/more
ln -sf xxd $DESTDIR/bin/hexdump
cp -f $SRC/etc/inputrc $DESTDIR/etc

for i in lib/* bin/* sbin/* lib/gconv/*.so; do
	strip -R .comment --strip-unneeded $i 2>/dev/null
done
cp -f $SRC/boot/rescue.img $DESTDIR/boot/rescue.img

cd $DESTDIR/..

chown -R root.root $DESTDIR

if [ -n "$WITH_CRAMFS" ]; then
	mkdir ${DESTDIR}.cram
	cp -avr $DESTDIR/boot $DESTDIR/README ${DESTDIR}.cram
	mkcramfs $DESTDIR ${DESTDIR}.cram/cramfs.img
	rm -fr $DESTDIR
	mv ${DESTDIR}.cram $DESTDIR
fi

mkisofs -quiet -A "Red Hat SysAdmin Survival CD $VERSION" -p "Red Hat GmbH" -R -b boot/rescue.img -c boot/boot.cat -hide-rr-moved -o rescue-cd-$VERSION.iso CD-IMAGE
